#!/usr/bin/python
# -*- coding: utf-8 -*-
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
__author__ = 'clarkmatthew'

'''
Data deduplication removes duplicate blocks, storing only unique blocks in the flex volume,
and it creates a small amount of additional metadata in the process.
 Data compression is a software-based solution that provides transparent data compression.
 Deduplication is a process that can be scheduled to run when it is most convenient,
 while data compression has the ability to run either as an inline process as data is written to disk or as a scheduled process.
 When the two are enabled on the same volume, the data is first compressed and then deduplicated.
'''

from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase
from san_client import San_Client
from eutester.euvolume import EuVolume
import time
import types
import re




class San_Properties_Test(EutesterTestCase):

    def __init__(self, san_ip, san_user, san_password, santype=None, tester=None, **kwargs):
        #### Pre-conditions
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("--santype",
                                 dest="santype",
                                 help="String representing san type. Default='netapp'",
                                 default='netapp')


        self.tester = tester
        self.get_args()
        # Allow __init__ to get args from __init__'s kwargs or through command line parser...
        for kw in kwargs:
            print 'Setting kwarg:'+str(kw)+" to "+str(kwargs[kw])
            self.set_arg(kw ,kwargs[kw])
        self.show_args()
        #if self.args.config:
        #    setattr(self.args, 'config_file',self.args.config)
        # Setup basic eutester object
        if not self.tester:
            try:
                self.tester = self.do_with_args(Eucaops)
            except Exception, e:
                raise Exception('Couldnt create Eucaops tester object, make sure credpath, ' \
                                'or config_file and password was provided, err:' + str(e))
            #replace default eutester debugger with eutestcase's for more verbosity...
            self.tester.debug = lambda msg: self.debug(msg, traceback=2, linebyline=False)
        self.tester.update_property_manager()
        self.san_ip = san_ip
        self.san_user = san_user
        self.san_password = san_password
        self.san = None
        self.zone = self.args.zone
        self.santype = santype or self.args.santype
        self.instance = None
        if self.args.zone:
            self.zone = str(self.args.zone)
        else:
            self.zone = 'PARTI00'
            self.groupname = 'san_property_tests'

        self.group = self.tester.add_group(self.groupname)
        self.tester.authorize_group(self.group)
        self.tester.authorize_group(self.group, protocol='icmp',port='-1')
        ### Generate a keypair for the instance
        if self.instance_password:
            self.keypair = None
        else:
            try:
                keys = self.tester.get_all_current_local_keys()
                if keys:
                    self.keypair = keys[0]
                else:
                    self.keypair = self.tester.add_keypair('sanpropertytestskey'+str(time.time()))
            except Exception, ke:
                raise Exception("Failed to find/create a keypair, error:" + str(ke))
        self.get_san_client()



    def get_san_client(self):
        self.san = san_client.get_san_client_by_type(host=self.host,
                                                             username=self.username,
                                                             password=self.password,
                                                             santype=self.santype)
        return self.san


    def update_volume_san_info(self, volume):
        if hasattr(volume, 'san_info'):
            volume.san_info.update()
        else:
            volume.san_info = self.san.get_volume_info_by_id(volume.id)


    def test_property_storage_deduppolicy(self):
        policy_property = self.tester.property_manager.get_property('deduppolicy', 'storage', self.zone)
        self.debug('Description: ' + str(policy_property.description))
        self.debug('Property before:' + str(policy_property.value))
        policy_before = policy_property.value
        enabled_property = self.tester.property_manager.get_property('', 'storage', self.zone)
        enabled_before = enabled_property.value
        enabled_property.set('true')

        property.print_self(print_method=self.debug)
        property.set('euca-policy')
        volume = self.tester.create



    def test_property_storage_dedupschedule(self):
        pass

    def test_property_storage_enablecompression(self):
        pass

    def test_property_storage_enablededup(self):
        pass

    def test_property_storage_enableinlinecompression(self):
        pass




#!/usr/bin/python
import unittest
import time
from eucaops import Eucaops
from eutester import xmlrunner
import os
import re
import random
import argparse

class LoadGenerator(unittest.TestCase):
    def setUp(self):
        # Setup basic eutester object
        self.tester = Eucaops( config_file="../input/2b_tested.lst", password="foobar")
        self.tester.poll_count = 40
        
        ### Determine whether virtio drivers are being used
        self.device_prefix = "sd"
        if self.tester.get_hypervisor() == "kvm":
            self.device_prefix = "vd"
        self.ephemeral = "/dev/" + self.device_prefix + "a2"
        
        ### Add and authorize a group for the instance
        zones = self.tester.ec2.get_all_zones()
        self.zone = random.choice(zones).name

    
    def tearDown(self):
        self.reservation = None
        self.group = None
        self.keypair = None
        self.tester = None
        self.ephemeral = None
    
    def GenerateKeypairs(self, count=10):
        """
        Create and delete keypairs in series
        """
        for i in xrange(count):
            key_name = "key-generator-" + str(i)
            keypair = self.tester.add_keypair()
            self.tester.delete_keypair(keypair)
     
    def GenerateVolumes(self, count=10):
        """
        Create and delete volumes in series
        """
        for i in xrange(count):
            volume = self.tester.create_volume(self.zone)
            self.tester.delete_volume(volume)

        
if __name__ == "__main__":
    ## If given command line arguments, use them as test names to launch
    parser = argparse.ArgumentParser(description='Parse test suite arguments.')
    parser.add_argument('--xml', action="store_true", default=False)
    parser.add_argument('--tests', nargs='+', default= ["GenerateKeypairs"])
    args = parser.parse_args()
    for test in args.tests:
        if args.xml:
            file = open("test-" + test + "result.xml", "w")
            result = xmlrunner.XMLTestRunner(file).run(LoadGenerator(test))
        else:
            result = unittest.TextTestRunner(verbosity=2).run(LoadGenerator(test))
        if result.wasSuccessful():
            pass
        else:
            exit(1)


__author__ = 'clarkmatthew'

import sys
import time
from eutester.eutestcase import EutesterTestCase
from eucaops import Eucaops
from eucaops.ec2ops import ResourceNotFoundException
from testcases.cloud_user.images.imageutils import ImageUtils





class Windows_Basic_Instance_Test(EutesterTestCase):

    def __init__(self, tester=None, **kwargs):
        #### Pre-conditions
        self.setuptestcase()
        self.setup_parser(testname='Windows_Basic_Instance_Test')
        self.parser.add_argument('--image_id',
                                 help='Image id of an existing image to run'
                                      ' test against, default:None',
                                 default=None)
        self.parser.add_argument('--image_url',
                                 help='Image url. Will look for an existing'
                                      'tag containing this url if not found'
                                      'will attempt to create an image from'
                                      'this url, default:None',
                                 default=None)
        self.parser.add_argument('--dont_clean',
                                action='store_true', default=False,
                                help='Boolean, will run all test methods in testsuite()')
        self.parser.add_argument('--auth_ports',
                                 help='Comma delimited list of "protocol:port'
                                      ' to authorize for this test, '
                                      'default: tcp:3389,tcp:80,tcp:443,tcp:5985,tcp:5986',
                                 default='tcp:3389,tcp:80,tcp:443,tcp:5985,tcp:5986')
        self.parser.add_argument('--image_run_timeout',
                                 help='Time to wait in seconds for an image to'
                                      ' go to running state. Default allows 60'
                                      ' seconds per 1 gig of image size, or '
                                      '600 seconds if size not available.',
                                 default=None)

        self.tester = tester
        self.get_args()
        # Allow __init__ to get args from __init__'s kwargs or through command line parser...
        for kw in kwargs:
            print 'Setting kwarg:'+str(kw)+" to "+str(kwargs[kw])
            self.set_arg(kw ,kwargs[kw])
        self.show_args()
        #if self.args.config:
        #    setattr(self.args, 'config_file',self.args.config)
        # Setup basic eutester object
        if not self.tester:
            try:
                self.tester = self.do_with_args(Eucaops)
            except Exception, e:
                raise Exception('Couldnt create Eucaops tester object, make sure credpath, '
                                'or config_file and password was provided, err:' + str(e))
        self.wins = None
        self.test_tag = 'windows_basic_instance_test'
        ### Add and authorize a group for the instance
        if self.args.zone:
            self.zone = str(self.args.zone)
        else:
            zones = self.tester.get_zones()
            if 'PARTI00' in zones:
                self.zone = 'PARTI00'
            elif zones:
                self.zone = zones[0]
            else:
                raise RuntimeError('No Zones found to run test')
        self.groupname = 'windows_basic_instance_test'
        self.group = self.tester.add_group(self.groupname)
        #For this basic test make it wide open...
        self.authorization_windows_group()
        ### Generate a keypair for the instance
        try:
            keys = self.tester.get_all_current_local_keys()
            if keys:
                self.keypair = keys[0]
            else:
                self.keypair = self.tester.add_keypair('mpathtestinstancekey'
                                                       + str(time.time()))
        except Exception, ke:
            raise Exception("Failed to find/create a keypair, error:" + str(ke))


    def authorization_windows_group(self,portlist=None):
        '''
        Where format for adding port in list is: 'protocol:port', example: tcp, port 80 = 'tcp:80'
        '''
        authports = portlist or self.args.auth_ports.split(',')
        for p in authports:
            protocol,port = str(p).split(':')
            self.tester.authorize_group_by_name(self.group.name,protocol=str(protocol),port=int(port))

    def get_a_windows_test_image(self):
        # If an image id was provided, attempt to get that image to use
        if self.args.image_id:
            self.image = self.tester.get_emi(self.args.image_id)
        else:
            # See if a eutester windows image already exists on the system
            # If an image url was provided try that first...
            if self.args.image_url:
                try:
                    self.image = self.tester.get_emi(
                        filters={'tag-value':self.args.image_url,
                                 'platform':'windows'})
                except ResourceNotFoundException:
                    self.debug('Image not found for provide url:"{0}", '
                               'attempting to create one now...'
                               .format(self.args.image_url))
                    self.create_image_from_url(self.args.image_url)
            else:
                # Otherwise look for a eutester created windows image...
                self.image = self.tester.get_emi(
                    filters={'tag-key':'eutester-created',
                             'platform':'windows'})
        self.debug('Using the following image to run this test. '
                   'ID:{0}, Name:{1}'.format(self.image.id, self.image.name))

    def create_image_from_url(self, url=None):
        """
        Definition: Attempts to create a windows image from a provided URL
        """
        url = url or self.args.image_url
        if not url:
            raise ValueError('No url provided to "create_image_from_url"')
        self.debug('Attempting to create image from:' + str(url))
        image_utils = ImageUtils(tester=self.tester)
        image = image_utils.create_emi(url=url,
                                       platform='windows',
                                       kernel='windows')
        self.status('Setting launch permissions to "all" on image:' + str(image.id))
        image.set_launch_permissions(group_names=['all'])
        self.debug('created image:' + str(image.id))
        self.image = image
        return image

    def get_min_vm_type_for_size(self, disk_size):
        for vmtype in self.tester.get_vm_type_list_from_zone(self.zone):
            if vmtype.disk >= disk_size:
                return vmtype.name
        raise RuntimeError('Could not find vmtype large enough for disk '
                           'size:' + str(disk_size))

    def run_windows_instance(self):
        """
        Definition: Attempts to run a windows instance.
        -Monitors instance to running state
        -Polls windows management ports for access/status
        -Attempts winrm connection to validate vm status.
        -Dumps system info,
         and disk info for block device mapping (ebs, ephemeral, etc)
        """
        vmtype = self.args.vmtype
        timeout = self.args.image_run_timeout
        default_vmtype = 'm2.xlarge'
        if not vmtype:
            size = None
            if 'size' in self.image.tags:
                if self.image.tags['size']:
                    size = int(self.image.tags['size'])
            if size:
                vmtype = self.get_min_vm_type_for_size(size)
                if not timeout:
                    timeout = 60 * size
            else:
                self.debug('No size found, using default vmtype:'
                           + str(default_vmtype))
                vmtype=default_vmtype
        if not timeout:
            timeout = 600
        if not self.image:
            self.get_a_windows_test_image()
        image_info = "Running the following Windows Image:\n"
        for key in self.image.__dict__:
            image_info += str(key) + " ----> " + \
                          str(self.image.__dict__[key]) + "\n"
        self.debug(image_info)
        self.wins = self.tester.run_image(image=self.image,
                                          group=self.group,
                                          keypair=self.keypair,
                                          zone=self.zone,
                                          type=vmtype,
                                          timeout=timeout)[0]
        return self.wins

    def check_ephemeral_test(self):
        """
        Definition: Test attempts to verify the ephemeral disk size for an
        instance store backed instance based upon the vmtype ran.
        """
        if self.wins is None:
            self.skipTest('No Windows instance to run test against')
        main_disk = None
        ephemeral_disk = None
        vmtype = self.tester.get_vm_type_from_zone(self.zone,
                                                   self.wins.instance_type)
        disk_size = vmtype.disk
        for disk in self.wins.diskdrives:
            if disk.index == 0:
                main_disk = disk
            if disk.index == 1:
                ephemeral_disk = disk
            if main_disk and ephemeral_disk:
                break
        self.debug(str(self.id) + ', Primary disk size:' +
                   str(main_disk.size))
        self.debug(str(self.id) + ', Ephemeral disk size:' +
                   str(main_disk.size))
        assert disk_size == main_disk.get_size_in_gb() + \
               ephemeral_disk.get_size_in_gb()


    def ebs_attach_detach_volume_test(self):
        '''
        Definition: Basic attach and detach ebs volume test.
        -Verifies ebs attached and detached states.
        -Verifies the volume appears on the guest as a new disk when attached
        -Verifies the size is correct
        -Verifies the volume is removed from the guest upon detach
        '''
        if self.wins is None:
            self.skipTest('No Windows instance to run test against')
        self.testvolume = self.tester.create_volume(zone=self.zone, size=1)
        self.wins.attach_volume(self.testvolume)
        time.sleep(10)
        self.wins.detach_euvolume(self.testvolume)


    def clean_method(self):
        self.tester.test_resources['images'] = []
        if self.args.dont_clean:
            self.debug('Not cleaning due to dont_clean arg')
        else:
            return self.tester.cleanup_artifacts()

if __name__ == "__main__":
    testcase = Windows_Basic_Instance_Test()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list
    test_names = testcase.args.tests or []
    if not test_names:
        testcase.get_a_windows_test_image()
        if not testcase.image:
            if not testcase.args.image_url:
                raise RuntimeError('No image found and no image url provided')
            else:
                test_names.append('create_image_from_url')
        test_names.extend(['run_windows_instance',
                          'check_ephemeral_test',
                          'ebs_attach_detach_volume_test'])
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in test_names:
        unit_list.append(testcase.create_testunit_by_name(test))

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,
                                         eof=False,
                                         clean_on_exit=True)
    sys.exit(result)






# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2014, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com
import re
import copy
import time
import datetime
from boto.ec2.regioninfo import RegionInfo
import boto.ec2.cloudwatch
from eutester.euinstance import EuInstance
from eutester import Eutester


CWRegionData =        {
                      'us-east-1': 'monitoring.us-east-1.amazonaws.com',
                      'us-west-1': 'monitoring.us-west-1.amazonaws.com',
                      'eu-west-1': 'monitoring.eu-west-1.amazonaws.com',
                      'ap-northeast-1': 'monitoring.ap-northeast-1.amazonaws.com',
                      'ap-southeast-1': 'monitoring.ap-southeast-1.amazonaws.com'
                      }

DimensionArray      = ['AutoScalingGroupName', 'ImageId', 'InstanceId', 'InstanceType']

StatsArray          = ['Average', 'Sum', 'Maximum', 'Minimum','SampleCount']

ComparisonOperator  = ['>=', '>', '<', '<=']

InstanceMetricArray = [
                      {'name':'CPUUtilization','unit':'Percent' },
                      {'name':'DiskReadOps','unit':'Count'},
                      {'name':'DiskWriteOps','unit':'Count'},
                      {'name':'DiskReadBytes','unit': 'Bytes'},
                      {'name':'DiskWriteBytes','unit':'Bytes'},
                      {'name':'NetworkIn','unit':'Bytes'},
                      {'name':'NetworkOut','unit':'Bytes'}
                      ]

StatusMetricArray   = [
                      {'name':'StatusCheckFailed','unit':'Count'},
                      {'name':'StatusCheckFailed_Instance','unit':'Count'},
                      {'name':'StatusCheckFailed_System','unit':'Count'}
                      ]
EbsMetricsArray     = [
                      {'name':'VolumeReadBytes','unit':'Bytes'},
                      {'name':'VolumeWriteBytes','unit':'Bytes'},
                      {'name':'VolumeReadOps','unit':'Count'},
                      {'name':'VolumeWriteOps','unit':'Count'},
                      {'name':'VolumeTotalReadTime','unit':'Seconds'},
                      {'name':'VolumeTotalWriteTime','unit':'Seconds'},
                      {'name':'VolumeIdleTime','unit':'Seconds'},
                      {'name':'VolumeQueueLength','unit':'Count'},
                      {'name':'VolumeThroughputPercentage','unit':'Percent'},
                      {'name':'VolumeConsumedReadWriteOps','unit':'Count'}
                      ]

class CWops(Eutester):
    @Eutester.printinfo
    def __init__(self, host=None, credpath=None, endpoint=None, aws_access_key_id=None, aws_secret_access_key=None,
                 username='root', region=None, is_secure=False, path='/', port=80, boto_debug=0):
        '''

        :param host:
        :param credpath:
        :param endpoint:
        :param aws_access_key_id:
        :param aws_secret_access_key:
        :param username:
        :param region:
        :param is_secure:
        :param path:
        :param port:
        :param boto_debug:
        '''
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.account_id = None
        self.user_id = None
        super(CWops, self).__init__(credpath=credpath)

        self.setup_cw_connection(host=host,
                                 region=region,
                                 endpoint=endpoint,
                                 aws_access_key_id=self.aws_access_key_id,
                                 aws_secret_access_key=self.aws_secret_access_key,
                                 is_secure=is_secure,
                                 path=path,
                                 port=port,
                                 boto_debug=boto_debug)
        self.poll_count = 48
        self.username = username
        self.test_resources = {}
        self.setup_cw_resource_trackers()

    @Eutester.printinfo
    def setup_cw_connection(self, endpoint=None, aws_access_key_id=None, aws_secret_access_key=None, is_secure=True,
                            host=None,
                            region=None, path='/', port=443, boto_debug=0):
        '''

        :param endpoint:
        :param aws_access_key_id:
        :param aws_secret_access_key:
        :param is_secure:
        :param host:
        :param region:
        :param path:
        :param port:
        :param boto_debug:
        :raise:
        '''
        cw_region = RegionInfo()
        if region:
            self.debug('Check region: ' + str(region))
            try:
                if not endpoint:
                    cw_region.endpoint = CWRegionData[region]
                else:
                    cw_region.endpoint = endpoint
            except KeyError:
                raise Exception('Unknown region: %s' % region)
        else:
            cw_region.name = 'eucalyptus'
            if not host:
                if endpoint:
                    cw_region.endpoint = endpoint
                else:
                    cw_region.endpoint = self.get_cw_ip()
        connection_args = {'aws_access_key_id': aws_access_key_id,
                           'aws_secret_access_key': aws_secret_access_key,
                           'is_secure': is_secure,
                           'debug': boto_debug,
                           'port': port,
                           'path': path,
                           'region': cw_region}

        if re.search('2.6', boto.__version__):
            connection_args['validate_certs'] = False

        try:
            cw_connection_args = copy.copy(connection_args)
            cw_connection_args['path'] = path
            cw_connection_args['region'] = cw_region
            self.debug('Attempting to create cloud watch connection to ' + cw_region.endpoint + ':' + str(port) + path)
            self.cw = boto.connect_cloudwatch(**cw_connection_args)
        except Exception, e:
            self.critical('Was unable to create Cloud Watch connection because of exception: ' + str(e))

    def setup_cw_resource_trackers(self):
        '''
        Setup keys in the test_resources hash in order to track artifacts created
        '''
        self.test_resources['alarms'] = []
        self.test_resources['metric'] = []
        self.test_resources['datapoint'] = []

    def get_cw_ip(self):
        '''Parse the eucarc for the AWS_CLOUDWATCH_URL'''
        cw_url = self.parse_eucarc('AWS_CLOUDWATCH_URL')
        return cw_url.split('/')[2].split(':')[0]

    def get_cw_path(self):
        """Parse the eucarc for the AWS_CLOUDWATCH_URL"""
        cw_url = self.parse_eucarc("AWS_CLOUDWATCH_URL")
        cw_path = "/".join(cw_url.split("/")[3:])
        return cw_path

    def get_namespaces(self):
        '''
        Convenience function for easily segregating metrics into their namespaces

        :return: Dict where key is the Namespace and the value is a list with all metrics
        '''
        metrics= self.cw.list_metrics()
        namespaces = {}
        for metric in metrics:
            if not namespaces[metric.namespace]:
                namespaces[metric.namespace] = [metric]
            else:
                namespaces[metric.namespace].append(metric)
        return namespaces

    def list_metrics( self, next_token=None, dimensions=None, metric_name=None, namespace=None ):
        self.debug('Calling list_metrics( {p1}, {p2}, {p3}, {p4} )'.format(p1=next_token, p2=dimensions, p3=metric_name, p4=namespace))
        return self.cw.list_metrics(next_token , dimensions, metric_name, namespace)

    def get_metric_statistics( self, period, start_time, end_time, metric_name, namespace, statistics, dimensions=None, unit=None):
        self.debug('Calling get_metric_statistics( {p1}, {p2}, {p3}, {p4}, {p5}, {p6}, {p7}, {p8} )'.format(
                   p1=period, p2=start_time, p3=end_time, p4=metric_name, p5=namespace, p6=statistics, p7=dimensions, p8=unit))
        return self.cw.get_metric_statistics(period, start_time, end_time, metric_name, namespace, statistics, dimensions, unit)

    def put_metric_data( self, namespace, name, value=None, timestamp=None, unit=None, dimensions=None, statistics=None):
        self.debug('Calling put_metric_data( {p1}, {p2}, {p3}, {p4}, {p5}, {p6}, {p7} )'.format(
                   p1=namespace, p2=name, p3=value, p4=timestamp, p5=unit, p6=dimensions, p7=dimensions))
        return self.cw.put_metric_data(namespace, name, value, timestamp, unit, dimensions, statistics)

    def metric_alarm(self, name, metric, comparison, threshold, period, evaluation_periods, statistic,
                     description=None, dimensions=None, alarm_actions=None,
                     ok_actions=None, insufficient_data_actions=None, unit=None, namespace=None):

        self.debug('Calling create_metric_alarm ( {p1}, {p2}, {p3}, {p4}, {p5}, {p6}, {p7}, {p8}, {p9}, {p10}, {p11}, {p12}, {p13}, {p14} )'.format(
                   p1=name, p2=metric, p3=comparison, p4=threshold, p5=period, p6=evaluation_periods, p7=statistic,
                   p8=description, p9=dimensions, p10=alarm_actions, p11=ok_actions, p12=insufficient_data_actions, p13=unit, p14=namespace))

        alarm = boto.ec2.cloudwatch.alarm.MetricAlarm(name=name, metric=metric, comparison=comparison, threshold=threshold, period=period,
                                                     evaluation_periods=evaluation_periods, statistic=statistic,
                                                     description=description, dimensions=dimensions, alarm_actions=alarm_actions,
                                                     ok_actions=ok_actions, insufficient_data_actions=insufficient_data_actions, unit=unit,
                                                     namespace=namespace)
        return alarm

    def put_metric_alarm(self, alarm):
        self.debug('Calling put_metric_alarm (' + str(alarm) +')')
        self.cw.put_metric_alarm(alarm)

    def set_alarm_state(self, alarm_name, state_reason='testing', state_value='ALARM', state_reason_data=None):
        self.debug('Calling set_alarm_state( {p1}, {p2}, {p3}, {p4})'.format( p1=alarm_name, p2=state_reason, p3=state_value, p4=state_reason_data))
        self.cw.set_alarm_state(alarm_name, state_reason, state_value)

    def delete_all_alarms(self):
        self.debug('Calling delete_all_alarms(' + str(self.cw.describe_alarms()) + ')')
        alarms = self.cw.describe_alarms()
        if alarms:
            alarm_names = [alarm.name for alarm in alarms]
            self.cw.delete_alarms(alarm_names)

    def describe_alarms(self, action_prefix=None, alarm_name_prefix=None, alarm_names=None, max_records=None, state_value=None, next_token=None):
        self.debug('Calling describe_alarms( {p1}, {p2}, {p3}, {p4}, {p5}, {p6} )'.format(p1=action_prefix, p2=alarm_name_prefix, p3=alarm_names,
                   p4=max_records, p5=state_value, p6=next_token))
        return self.cw.describe_alarms(action_prefix, alarm_name_prefix, alarm_names, max_records, state_value, next_token)

    def describe_alarms_for_metric(self, metric_name, namespace, period=None, statistic=None, dimensions=None, unit=None):
        self.debug('Calling describe_alarms_for_metric( {p1}, {p2}, {p3}, {p4}, {p5}, {p6} )'.format(p1=metric_name, p2=namespace, p3=period, p4=statistic,
                   p5=dimensions, p6=unit))
        return self.cw.describe_alarms_for_metric(metric_name, namespace, period, statistic, dimensions, unit)

    def describe_alarm_history(self, alarm_name=None, start_date=None, end_date=None, max_records=None, history_item_type=None, next_token=None):
        self.debug('Calling describe_alarm_history( {p1}, {p2}, {p3}, {p4}, {p5}, {p6} )'.format(p1=alarm_name, p2=start_date, p3=end_date, p4=max_records, 
                  p5=history_item_type, p6=next_token))
        return self.cw.describe_alarm_history(alarm_name, start_date, end_date, max_records, history_item_type, next_token)

    def get_dimension_array(self):
        return DimensionArray

    def get_stats_array(self):
        return StatsArray

    def get_instance_metrics_array(self):
        return InstanceMetricArray

    def get_status_metric_array(self):
        return StatusMetricArray

    def get_ebs_metrics_array(self):
        return EbsMetricsArray

    def enable_alarm_actions(self, alarm_names ):
        self.debug('Calling enable_alarm_actions( ' + str(alarm_names) + ' )')
        self.cw.enable_alarm_actions(alarm_names)

    def disable_alarm_actions(self, alarm_names ):
        self.debug('Calling disable_alarm_actions( ' + str(alarm_names) + ' )')
        self.cw.disable_alarm_actions(alarm_names)

    def validateStats(self, values):
        average = float(values[0])
        theSum  = float(values[1])
        maximum = float(values[2])
        minimum = float(values[3])
        sample = float(values[4])
        assert average <= maximum and average >= minimum
        assert maximum >= minimum
        assert minimum <= maximum
        assert sample > 0




#!/usr/bin/env python
#
#
# Description:  This script encompasses test cases/modules concerning instance specific behavior and
#               features for Eucalyptus.  The test cases/modules that are executed can be 
#               found in the script under the "tests" list.

import time
from concurrent.futures import ThreadPoolExecutor
import threading
from eucaops import Eucaops
from eutester.euinstance import EuInstance
from eutester.eutestcase import EutesterTestCase
from eucaops import EC2ops
import os
import re
import random


class InstanceBasics(EutesterTestCase):
    def __init__( self, name="InstanceBasics", credpath=None, region=None, config_file=None, password=None, emi=None, zone=None,
                  user_data=None, instance_user=None, **kwargs):
        """
        EC2 API tests focused on instance store instances

        :param credpath: Path to directory containing eucarc file
        :param region: EC2 Region to run testcase in
        :param config_file: Configuration file path
        :param password: SSH password for bare metal machines if config is passed and keys arent synced
        :param emi: Image id to use for test
        :param zone: Availability Zone to run test in
        :param user_data: User Data to pass to instance
        :param instance_user: User to login to instance as
        :param kwargs: Additional arguments
        """
        super(InstanceBasics, self).__init__(name=name)
        if region:
            self.tester = EC2ops(credpath=credpath, region=region)
        else:
            self.tester = Eucaops(config_file=config_file, password=password, credpath=credpath)
        self.instance_timeout = 600

        ### Add and authorize a group for the instance
        self.group = self.tester.add_group(group_name="group-" + str(time.time()))
        self.tester.authorize_group_by_name(group_name=self.group.name)
        self.tester.authorize_group_by_name(group_name=self.group.name, port=-1, protocol="icmp")
        ### Generate a keypair for the instance
        self.keypair = self.tester.add_keypair("keypair-" + str(time.time()))
        self.keypath = '%s/%s.pem' % (os.curdir, self.keypair.name)
        if emi:
            self.image = emi
        else:
            self.image = self.tester.get_emi(root_device_type="instance-store", not_platform="windows")
        self.address = None
        self.volume = None
        self.private_addressing = False
        if not zone:
            zones = self.tester.ec2.get_all_zones()
            self.zone = random.choice(zones).name
        else:
            self.zone = zone
        self.reservation = None
        self.reservation_lock = threading.Lock()
        self.run_instance_params = {'image': self.image, 'user_data': user_data, 'username': instance_user,
                                    'keypair': self.keypair.name, 'group': self.group.name, 'zone': self.zone,
                                    'timeout': self.instance_timeout}
        self.managed_network = True

        ### If I have access to the underlying infrastructure I can look
        ### at the network mode and only run certain tests where it makes sense
        if hasattr(self.tester, "service_manager"):
            cc = self.tester.get_component_machines("cc")[0]
            network_mode = cc.sys("cat " + self.tester.eucapath + "/etc/eucalyptus/eucalyptus.conf | grep MODE")[0]
            if re.search("(SYSTEM|STATIC)", network_mode):
                self.managed_network = False

    def set_reservation(self, reservation):
        self.reservation_lock.acquire()
        self.reservation = reservation
        self.reservation_lock.release()

    def clean_method(self):
        self.tester.cleanup_artifacts()

    def BasicInstanceChecks(self):
        """
        This case was developed to run through a series of basic instance tests.
             The tests are as follows:
                   - execute run_instances command
                   - make sure that public DNS name and private IP aren't the same
                       (This is for Managed/Managed-NOVLAN networking modes)
                   - test to see if instance is ping-able
                   - test to make sure that instance is accessible via ssh
                       (ssh into instance and run basic ls command)
             If any of these tests fail, the test case will error out, logging the results.
        """
        reservation = self.tester.run_instance(**self.run_instance_params)
        for instance in reservation.instances:
            self.assertTrue(self.tester.wait_for_reservation(reservation), 'Instance did not go to running')
            self.assertTrue(self.tester.ping(instance.ip_address), 'Could not ping instance')
            if self.image.virtualization_type == "paravirtual":
                paravirtual_ephemeral = "/dev/" + instance.rootfs_device + "2"
                self.assertFalse(instance.found("ls -1 " + paravirtual_ephemeral,  "No such file or directory"),
                                 "Did not find ephemeral storage at " + paravirtual_ephemeral)
            elif self.image.virtualization_type == "hvm":
                hvm_ephemeral = "/dev/" + instance.block_device_prefix + "b"
                self.assertFalse(instance.found("ls -1 " + hvm_ephemeral,  "No such file or directory"),
                                 "Did not find ephemeral storage at " + hvm_ephemeral)
        self.set_reservation(reservation)
        return reservation

    def ElasticIps(self):
        """
       This case was developed to test elastic IPs in Eucalyptus. This test case does
       not test instances that are launched using private-addressing option.
       The test case executes the following tests:
           - allocates an IP, associates the IP to the instance, then pings the instance.
           - disassociates the allocated IP, then pings the instance.
           - releases the allocated IP address
       If any of the tests fail, the test case will error out, logging the results.
        """
        if not self.reservation:
            reservation = self.tester.run_instance(**self.run_instance_params)
        else:
            reservation = self.reservation

        for instance in reservation.instances:
            if instance.ip_address == instance.private_ip_address:
                self.tester.debug("WARNING: System or Static mode detected, skipping ElasticIps")
                return reservation
            self.address = self.tester.allocate_address()
            self.assertTrue(self.address, 'Unable to allocate address')
            self.tester.associate_address(instance, self.address)
            instance.update()
            self.assertTrue(self.tester.ping(instance.ip_address), "Could not ping instance with new IP")
            self.tester.disassociate_address_from_instance(instance)
            self.tester.release_address(self.address)
            self.address = None
            assert isinstance(instance, EuInstance)
            self.tester.sleep(5)
            instance.update()
            self.assertTrue(self.tester.ping(instance.ip_address), "Could not ping after dissassociate")
        self.set_reservation(reservation)
        return reservation

    def MultipleInstances(self):
        """
        This case was developed to test the maximum number of m1.small vm types a configured
        cloud can run.  The test runs the maximum number of m1.small vm types allowed, then
        tests to see if all the instances reached a running state.  If there is a failure,
        the test case errors out; logging the results.
        """
        if self.reservation:
            self.tester.terminate_instances(self.reservation)
            self.set_reservation(None)

        reservation = self.tester.run_instance(min=2, max=2, **self.run_instance_params)
        self.assertTrue(self.tester.wait_for_reservation(reservation), 'Not all instances  went to running')
        self.set_reservation(reservation)
        return reservation

    def LargestInstance(self):
        """
        This case was developed to test the maximum number of c1.xlarge vm types a configured
        cloud can run.  The test runs the maximum number of c1.xlarge vm types allowed, then
        tests to see if all the instances reached a running state.  If there is a failure,
        the test case errors out; logging the results.
        """
        if self.reservation:
            self.tester.terminate_instances(self.reservation)
            self.set_reservation(None)
        reservation = self.tester.run_instance(type="c1.xlarge", **self.run_instance_params)
        self.assertTrue(self.tester.wait_for_reservation(reservation), 'Not all instances  went to running')
        self.set_reservation(reservation)
        return reservation

    def MetaData(self):
        """
        This case was developed to test the metadata service of an instance for consistency.
        The following meta-data attributes are tested:
           - public-keys/0/openssh-key
           - security-groups
           - instance-id
           - local-ipv4
           - public-ipv4
           - ami-id
           - ami-launch-index
           - reservation-id
           - placement/availability-zone
           - kernel-id
           - public-hostname
           - local-hostname
           - hostname
           - ramdisk-id
           - instance-type
           - any bad metadata that shouldn't be present.
        Missing nodes
         ['block-device-mapping/',  'ami-manifest-path']
        If any of these tests fail, the test case will error out; logging the results.
        """
        if not self.reservation:
            reservation = self.tester.run_instance(**self.run_instance_params)
        else:
            reservation = self.reservation
        for instance in reservation.instances:
            ## Need to verify  the public key (could just be checking for a string of a certain length)
            self.assertTrue(re.match(instance.get_metadata("public-keys/0/openssh-key")[0].split('eucalyptus.')[-1],
                                     self.keypair.name), 'Incorrect public key in metadata')
            self.assertTrue(re.match(instance.get_metadata("security-groups")[0], self.group.name),
                            'Incorrect security group in metadata')
            # Need to validate block device mapping
            #self.assertTrue(re.search(instance_ssh.get_metadata("block-device-mapping/")[0], "")) 
            self.assertTrue(re.match(instance.get_metadata("instance-id")[0], instance.id),
                            'Incorrect instance id in metadata')
            self.assertTrue(re.match(instance.get_metadata("local-ipv4")[0], instance.private_ip_address),
                            'Incorrect private ip in metadata')
            self.assertTrue(re.match(instance.get_metadata("public-ipv4")[0], instance.ip_address),
                            'Incorrect public ip in metadata')
            self.assertTrue(re.match(instance.get_metadata("ami-id")[0], instance.image_id),
                            'Incorrect ami id in metadata')
            self.assertTrue(re.match(instance.get_metadata("ami-launch-index")[0], instance.ami_launch_index),
                            'Incorrect launch index in metadata')
            self.assertTrue(re.match(instance.get_metadata("reservation-id")[0], reservation.id),
                            'Incorrect reservation in metadata')
            self.assertTrue(re.match(instance.get_metadata("placement/availability-zone")[0], instance.placement),
                            'Incorrect availability-zone in metadata')
            if self.image.virtualization_type == "paravirtual":
                self.assertTrue(re.match(instance.get_metadata("kernel-id")[0], instance.kernel),
                                'Incorrect kernel id in metadata')
                self.assertTrue(re.match(instance.get_metadata("ramdisk-id")[0], instance.ramdisk),
                                'Incorrect ramdisk in metadata')
            self.assertTrue(re.match(instance.get_metadata("public-hostname")[0], instance.public_dns_name),
                            'Incorrect public host name in metadata')
            self.assertTrue(re.match(instance.get_metadata("local-hostname")[0], instance.private_dns_name),
                            'Incorrect private host name in metadata')
            self.assertTrue(re.match(instance.get_metadata("hostname")[0], instance.private_dns_name),
                            'Incorrect host name in metadata')
            self.assertTrue(re.match(instance.get_metadata("instance-type")[0], instance.instance_type),
                            'Incorrect instance type in metadata')
            bad_meta_data_keys = ['foobar']
            for key in bad_meta_data_keys:
                self.assertTrue(re.search("Not Found", "".join(instance.get_metadata(key))),
                                'No fail message on invalid meta-data node')
        self.set_reservation(reservation)
        return reservation

    def DNSResolveCheck(self):
        """
        This case was developed to test DNS resolution information for public/private DNS
        names and IP addresses.  The tested DNS resolution behavior is expected to follow

        AWS EC2.  The following tests are ran using the associated meta-data attributes:
           - check to see if Eucalyptus Dynamic DNS is configured
           - nslookup on hostname; checks to see if it matches local-ipv4
           - nslookup on local-hostname; check to see if it matches local-ipv4
           - nslookup on local-ipv4; check to see if it matches local-hostname
           - nslookup on public-hostname; check to see if it matches local-ipv4
           - nslookup on public-ipv4; check to see if it matches public-host
        If any of these tests fail, the test case will error out; logging the results.
        """
        if not self.reservation:
            reservation = self.tester.run_instance(**self.run_instance_params)
        else:
            reservation = self.reservation

        for instance in reservation.instances:
            if not re.search("internal", instance.private_dns_name):
                self.tester.debug("Did not find instance DNS enabled, skipping test")
                self.set_reservation(reservation)
                return reservation
            # Test to see if Dynamic DNS has been configured #
            # Per AWS standard, resolution should have private hostname or private IP as a valid response
            # Perform DNS resolution against public IP and public DNS name
            # Perform DNS resolution against private IP and private DNS name
            # Check to see if nslookup was able to resolve
            assert isinstance(instance, EuInstance)
            # Check nslookup to resolve public DNS Name to local-ipv4 address
            self.assertTrue(instance.found("nslookup " + instance.public_dns_name,
                                           instance.private_ip_address), "Incorrect DNS resolution for hostname.")
            # Check nslookup to resolve public-ipv4 address to public DNS name
            if self.managed_network:
                self.assertTrue(instance.found("nslookup " + instance.ip_address,
                                               instance.public_dns_name),
                                "Incorrect DNS resolution for public IP address")
            # Check nslookup to resolve private DNS Name to local-ipv4 address
            if self.managed_network:
                self.assertTrue(instance.found("nslookup " + instance.private_dns_name,
                                               instance.private_ip_address),
                                "Incorrect DNS resolution for private hostname.")
            # Check nslookup to resolve local-ipv4 address to private DNS name
            self.assertTrue(instance.found("nslookup " + instance.private_ip_address,
                                           instance.private_dns_name),
                            "Incorrect DNS resolution for private IP address")
            self.assertTrue(self.tester.ping(instance.public_dns_name))
        self.set_reservation(reservation)
        return reservation

    def Reboot(self):
        """
        This case was developed to test IP connectivity and volume attachment after
        instance reboot.  The following tests are done for this test case:
                   - creates a 1 gig EBS volume, then attach volume
                   - reboot instance
                   - attempts to connect to instance via ssh
                   - checks to see if EBS volume is attached
                   - detaches volume
                   - deletes volume
        If any of these tests fail, the test case will error out; logging the results.
        """
        if not self.reservation:
            reservation = self.tester.run_instance(**self.run_instance_params)
        else:
            reservation = self.reservation
        for instance in reservation.instances:
            ### Create 1GB volume in first AZ
            volume = self.tester.create_volume(instance.placement, size=1, timepergig=180)
            instance.attach_volume(volume)
            ### Reboot instance
            instance.reboot_instance_and_verify(waitconnect=20)
            instance.detach_euvolume(volume)
            self.tester.delete_volume(volume)
        self.set_reservation(reservation)
        return reservation

    def Churn(self):
        """
        This case was developed to test robustness of Eucalyptus by starting instances,
        stopping them before they are running, and increase the time to terminate on each
        iteration.  This test case leverages the BasicInstanceChecks test case. The
        following steps are ran:
            - runs BasicInstanceChecks test case 5 times, 10 second apart.
            - While each test is running, run and terminate instances with a 10sec sleep in between.
            - When a test finishes, rerun BasicInstanceChecks test case.
        If any of these tests fail, the test case will error out; logging the results.
        """
        if self.reservation:
            self.tester.terminate_instances(self.reservation)
            self.set_reservation(None)
        try:
            available_instances_before = self.tester.get_available_vms(zone=self.zone)
            if available_instances_before > 4:
                count = 4
            else:
                count = available_instances_before
        except IndexError, e:
            self.debug("Running as non-admin, defaulting to 4 VMs")
            available_instances_before = count = 4

        future_instances = []

        with ThreadPoolExecutor(max_workers=count) as executor:
            ## Start asynchronous activity
            ## Run 5 basic instance check instances 10s apart
            for i in xrange(count):
                future_instances.append(executor.submit(self.BasicInstanceChecks))
                self.tester.sleep(10)

        with ThreadPoolExecutor(max_workers=count) as executor:
            ## Start asynchronous activity
            ## Terminate all instances
            for future in future_instances:
                executor.submit(self.tester.terminate_instances, future.result())

        def available_after_greater():
            return self.tester.get_available_vms(zone=self.zone) >= available_instances_before
        self.tester.wait_for_result(available_after_greater, result=True, timeout=360)

    def PrivateIPAddressing(self):
        """
        This case was developed to test instances that are launched with private-addressing
        set to True.  The tests executed are as follows:
            - run an instance with private-addressing set to True
            - allocate/associate/disassociate/release an Elastic IP to that instance
            - check to see if the instance went back to private addressing
        If any of these tests fail, the test case will error out; logging the results.
        """
        if self.reservation:
            for instance in self.reservation.instances:
                if instance.ip_address == instance.private_ip_address:
                    self.tester.debug("WARNING: System or Static mode detected, skipping PrivateIPAddressing")
                    return self.reservation
            self.tester.terminate_instances(self.reservation)
            self.set_reservation(None)
        reservation = self.tester.run_instance(private_addressing=True, **self.run_instance_params)
        for instance in reservation.instances:
            address = self.tester.allocate_address()
            self.assertTrue(address, 'Unable to allocate address')
            self.tester.associate_address(instance, address)
            self.tester.sleep(30)
            instance.update()
            self.assertTrue(self.tester.ping(instance.ip_address), "Could not ping instance with new IP")
            address.disassociate()
            self.tester.sleep(30)
            instance.update()
            self.assertFalse(self.tester.ping(instance.ip_address),
                             "Was able to ping instance that should have only had a private IP")
            address.release()
            if instance.ip_address != "0.0.0.0" and instance.ip_address != instance.private_ip_address:
                self.fail("Instance received a new public IP: " + instance.ip_address)
        self.tester.terminate_instances(reservation)
        self.set_reservation(reservation)
        return reservation

    def ReuseAddresses(self):
        """
        This case was developed to test when you run instances in a series, and make sure
        they get the same address.  The test launches an instance, checks the IP information,
        then terminates the instance. This test is launched 5 times in a row.  If there
        is an error, the test case will error out; logging the results.
        """
        prev_address = None
        if self.reservation:
            self.tester.terminate_instances(self.reservation)
            self.set_reservation(None)
        for i in xrange(5):
            reservation = self.tester.run_instance(**self.run_instance_params)
            for instance in reservation.instances:
                if prev_address is not None:
                    self.assertTrue(re.search(str(prev_address), str(instance.ip_address)),
                                    str(prev_address) + " Address did not get reused but rather  " +
                                    str(instance.public_dns_name))
                prev_address = instance.ip_address
            self.tester.terminate_instances(reservation)

    def BundleInstance(self):
        if not self.reservation:
            self.reservation = self.tester.run_instance(**self.run_instance_params)
        original_image = self.run_instance_params['image']
        for instance in self.reservation.instances:
            current_time = str(int(time.time()))
            temp_file = "/root/my-new-file-" + current_time
            instance.sys("touch " + temp_file)
            self.tester.sleep(60)
            starting_uptime = instance.get_uptime()
            self.run_instance_params['image'] = self.tester.bundle_instance_monitor_and_register(instance)
            instance.connect_to_instance()
            ending_uptime = instance.get_uptime()
            if ending_uptime > starting_uptime:
                raise Exception("Instance did not get stopped then started")
            bundled_image_reservation = self.tester.run_instance(**self.run_instance_params)
            for new_instance in bundled_image_reservation.instances:
                new_instance.sys("ls " + temp_file, code=0)
            self.tester.terminate_instances(bundled_image_reservation)
        self.run_instance_params['image'] = original_image

if __name__ == "__main__":
    testcase = EutesterTestCase(name='instancetest')
    testcase.setup_parser(description="Test the Eucalyptus EC2 instance store image functionality.")
    testcase.get_args()
    instancetestsuite = testcase.do_with_args(InstanceBasics)

    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    test_list = testcase.args.tests or ["BasicInstanceChecks", "DNSResolveCheck", "Reboot", "MetaData", "ElasticIps",
                                        "MultipleInstances", "LargestInstance", "PrivateIPAddressing", "Churn"]
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = []
    for test in test_list:
        test = getattr(instancetestsuite, test)
        unit_list.append(testcase.create_testunit_from_method(test))
    testcase.clean_method = instancetestsuite.clean_method
    result = testcase.run_test_case_list(unit_list)
    exit(result)



import unittest
from timer import Timer;

class TestTimer(unittest.TestCase):
    def setUp(self):
        pass

    def test_timer(self):
        t = Timer()
        id = t.start();
        id1 = t.start();
        print("wheeeeee")
        t.end(id, "mooo")
        t.end(id1, "fooo")
        t.finish()

if __name__ == '__main__':
    unittest.main()



# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com

from eucaops import Eucaops
import re
import time
import httplib
from xml.etree import ElementTree
import sys
from eutester.eutestcase import EutesterTestCase
from eutester.sshconnection import SshCbReturn
from eutester.machine import Machine
from conversiontask import ConversionTask


class ImageUtils(EutesterTestCase):
    #Define the bytes per gig
    gig = 1073741824
    mb = 1048576
    kb = 1024

    def __init__(self,
                 tester=None,
                 config_file=None,
                 password="foobar",
                 credpath=None,
                 destpath=None,
                 time_per_gig=300,
                 eof=True,
                 worker_hostname=None,
                 worker_keypath=None,
                 worker_username='root',
                 worker_password=None,
                 worker_machine=None):

        if tester is None:
            self.tester = Eucaops(config_file=config_file,
                                  password=password,
                                  credpath=credpath)
        else:
            self.tester = tester
        self.tester.exit_on_fail = eof
        self.debugmethod = self.tester.debug

        #Setup the work machine, this is the machine which will be used for
        # performing the 'work' (download, bundle, etc)
        self.worker_keypath = worker_keypath or self.tester.keypath
        self.worker_username = worker_username
        self.worker_password = worker_password or self.tester.password
        self.worker_machine = self._get_worker_machine(worker_machine)
        if destpath is not None:
            self.destpath = str(destpath)
        else:
            self.destpath = "/disk1/storage"

        self.time_per_gig = time_per_gig
        self.credpath = credpath or self.tester.credpath

    def _get_worker_machine(self, worker):
        '''
        Attempts to verify the worker passed is a Machine() class else assume
        it's a host name of the machine work should be performed on and
        attempt to return a Machine() created from the hostname

        param: worker Machine() or 'hostname' to be used to perform image utils
        work on.
        returns Machine obj
        '''
        worker = worker or self.tester.clc
        self.debug('Verifying ')
        if isinstance(worker, Machine):
            return worker
        else:
            #Assume this is a host addr, attempt to convert it to a Machine()
            self.debug('Attempting to connect to machine: ' + str(worker) +
                       " for image utility work...")
            new_machine = Machine(hostname=worker,
                                  username=self.worker_username,
                                  password=self.worker_password,
                                  keypath=self.worker_keypath)
            return new_machine

    def getHttpRemoteImageSize(self, url, unit=None):
            '''
            Get the remote file size from the http header of the url given
            Returns size in GB unless unit is given.
            '''
            unit = unit or self.__class__.gig
            try:
                url = url.replace('http://', '')
                host = url.split('/')[0]
                path = url.replace(host, '')
                self.debug("get_remote_file, host(" + host + ") path(" +
                           path + ")")
                conn = httplib.HTTPConnection(host)
                conn.request("HEAD", path)
                res = conn.getresponse()
                fbytes = int(res.getheader('content-length'))
                self.debug("content-length:" + str(fbytes))
                if fbytes == 0:
                    rfsize = 0
                else:
                    rfsize = (((fbytes/unit) + 1) or 1)
                self.debug("Remote file size: " + str(rfsize) + "g")
            except Exception, e:
                self.debug("Failed to get remote file size...")
                raise e
            finally:
                if conn:
                    conn.close()
            return rfsize

    def wget_image(self,
                   url,
                   destpath=None,
                   dest_file_name=None,
                   machine=None,
                   user=None,
                   password=None,
                   retryconn=True,
                   time_per_gig=300):
        '''
        Attempts to wget a url to a remote (worker) machine.
        :param url: url to wget/download
        :param destpath:path/dir to download to
        :param dest_file_name: filename to download image to
        :param machine: remote (worker) machine to wget on
        :param user: wget user name
        :param password: wget password
        :param retryconn: boolean to retry connection
        :param time_per_gig: int time to allow per gig of image wget'd
        :returns int size of image
        '''
        machine = machine or self.worker_machine
        if destpath is None and self.destpath is not None:
            destpath = self.destpath
        size = self.getHttpRemoteImageSize(url)
        if (size <  machine.get_available(destpath, unit=self.__class__.gig)):
            raise Exception("Not enough available space at: " +
                            str(destpath) + ", for image: " + str(url))
        timeout = size * time_per_gig
        self.debug('wget_image: ' + str(url) + ' to destpath' +
                   str(destpath) + ' on machine:' + str(machine.hostname))
        machine.wget_remote_image(url,
                                  path=destpath,
                                  dest_file_name=dest_file_name,
                                  user=user,
                                  password=password,
                                  retryconn=retryconn,
                                  timeout=timeout)
        return size


    def get_manifest_obj(self, path, machine=None, local=False, timeout=30):
        '''
        Read in a local or remote manifest xml file and convert to an
        xml ElementTree object.
        :param path: local or remote path to manifest file
        :param machine: the remote machine to read manifest file from
        :param local: boolean to determine if file is local
        :param timeout: timeout in second for reading in file
        :returns xml ElementTree obj
        '''
        cmd = 'cat ' + str(path)
        if not local:
            machine = machine or self.worker_machine
            out = machine.cmd(cmd, timeout=timeout, verbose=False)
            if out['status'] != 0:
                raise Exception('get_manifest_part_count failed, cmd status:'
                                + str(out['status']))
            output = out['output']
        else:
            output = self.tester.sys(cmd, timeout=timeout,
                                  listformat=False, code=0)
        xml = ElementTree.fromstring(output)
        return xml

    def get_manifest_part_count(self,
                                path,
                                machine=None,
                                local=False,
                                timeout=30):
        '''
        Attempt retrieve the part count value from a manifest file
        :param path: local or remote path to manifest file
        :param machine: the remote machine to read manifest file from
        :param local: boolean to determine if file is local
        :param timeout: timeout in second for reading in file
        :returns int count
        '''
        manifest_xml = self.get_manifest_obj(path=path,
                                             machine=machine,
                                             local=local,
                                             timeout=timeout)
        image = manifest_xml.find('image')
        parts = image.find('parts')
        part_count = parts.get('count')
        self.debug('get_manifest_part_count:' + str(path) +
                   ', count:' + str(part_count))
        return int(part_count)

    def get_manifest_image_name(self,
                                path,
                                machine=None,
                                local=False,
                                timeout=30):
        '''
        Attempts to read the image name from a manifest file
        :param path: local or remote path to manifest file
        :param machine: the remote machine to read manifest file from
        :param local: boolean to determine if file is local
        :param timeout: timeout in second for reading in file
        :returns string image name
        '''
        manifest_xml = self.get_manifest_obj(path=path,
                                             machine=machine,
                                             local=local,
                                             timeout=timeout)
        image = manifest_xml.find('image')
        name_elem = image.find('name')
        return name_elem.text
    
    def euca2ools_bundle_image(self,
                     path,
                     machine=None,
                     machine_credpath=None,
                     prefix=None,
                     kernel=None,
                     ramdisk=None,
                     block_device_mapping=None,
                     destination='/disk1/storage',
                     arch='x86_64',
                     debug=False,
                     interbundle_timeout=120, 
                     time_per_gig=None):
        '''
        Bundle an image on a 'machine'.
        where credpath to creds on machine
        '''
        time_per_gig = time_per_gig or self.time_per_gig
        credpath = machine_credpath or self.credpath
        machine = machine or self.worker_machine
        image_size = machine.get_file_size(path)/self.gig or 1
        timeout = time_per_gig * image_size
        cbargs = [timeout, interbundle_timeout, time.time(), 0, True]
        if destination is None:
            destination = machine.sys('pwd')[0]
        freesize = machine.get_available(str(destination), (self.gig/self.kb))
        if (freesize < image_size):
            raise Exception("Not enough free space at:"+str(destination))
        
        #build our tools bundle-image command...
        cmdargs = ""
        if prefix:
            cmdargs = cmdargs + " --prefix " +str(prefix)
        if kernel:
            cmdargs = cmdargs + " --kernel "  +str(kernel)
        if ramdisk:
            cmdargs = cmdargs + " --ramdisk " +str(ramdisk)
        if block_device_mapping:
            cmdargs = cmdargs + " --block-device-mapping " + \
                      str(block_device_mapping)
        if destination:
            cmdargs = cmdargs + " --destination " + str(destination)
        if arch:
            cmdargs = cmdargs + " --arch " + str(arch)
        if debug:
            cmdargs = cmdargs + " --debug "
        
        cmdargs = cmdargs + " -i " + str(path)

        if credpath is not None:
            cmd = 'source ' + str(credpath) + '/eucarc && euca-bundle-image ' \
                  + str(cmdargs)
        else:
            skey = self.tester.get_secret_key()
            akey = self.tester.get_access_key()
            cmd = 'euca-bundle-image -a ' + str(akey) + ' -s ' + \
                  str(skey) + str(cmdargs)
        #execute the command  
        out = machine.cmd(cmd, timeout=timeout, listformat=True,
                          cb = self._bundle_status_cb, cbargs=cbargs)
        if out['status'] != 0:
            raise Exception('bundle_image "' + str(path) +
                            '" failed. Errcode:' + str(out['status']))
        manifest = None
        for line in out['output']:
            line = str(line)
            if re.search("(Generating|Wrote) manifest",line):
                manifest = line.split()[2]
                break
        if manifest is None:
            raise Exception('Failed to find manifest from bundle_image:' +
                            str(path))
        self.debug('bundle_image:'+str(path)+'. manifest:'+str(manifest))
        return manifest

    def euca2ools_upload_bundle(self,
                      manifest, 
                      machine=None,
                      bucketname=None, 
                      machine_credpath=None,
                      debug=False, 
                      interbundle_timeout=120, 
                      timeout=0, 
                      image_check_timeout=300,
                      uniquebucket=True):
        '''
        Bundle an image on a 'machine'.
        where credpath to creds on machine
        '''
        machine = machine or self.worker_machine
        credpath = machine_credpath or self.credpath
        cbargs = [timeout, interbundle_timeout, time.time(), 0, True]
        bname = ''
        cmdargs = ""
        manifest = str(manifest)
        upmanifest = None
        part_count = -1
        try:
            part_count = self.get_manifest_part_count(manifest, machine=machine)
        except:
            pass
        self.debug('Attempting to upload_bundle:' + str(manifest) +
                   ", bucketname:" + str(bucketname) + ", part_count:" +
                   str(part_count))
        if bucketname:
            bname = bucketname
            if uniquebucket:
                bname = self._get_unique_bucket_name(bname)
        else:
            #Use the image name found in the manifest as bucketname
            bname = self._generate_unique_bucket_name_from_manifest(
                manifest, unique=uniquebucket)
        self.debug('Using upload_bundle bucket name: '+str(bname))
        cmdargs = cmdargs + " -b " +str(bname)
        if debug:
            cmdargs = cmdargs + " --debug "
        cmdargs = cmdargs + " -b " + str(bname) + " -m " +str(manifest)
        if credpath is not None:
            cmd = 'source ' + str(credpath) + '/eucarc && euca-upload-bundle '\
                  + str(cmdargs)
        else:
            skey = self.tester.get_secret_key()
            akey = self.tester.get_access_key()
            cmd = 'euca-upload-bundle -a '+str(akey) + ' -s ' + \
                  str(skey) + str(cmdargs)
        #execute upload-bundle command...
        out = machine.cmd(cmd, timeout=image_check_timeout, listformat=True,
                          cb=self._bundle_status_cb, cbargs=cbargs)
        if out['status'] != 0:
            raise Exception('upload_bundle "' + str(manifest) +
                            '" failed. Errcode:' + str(out['status']))
        for line in out['output']:
            line = str(line)
            if re.search('Uploaded', line) and re.search('manifest', line):
                upmanifest = line.split().pop()
                break
        if upmanifest is None:
            raise Exception('Failed to find upload manifest from '
                            'upload_bundle command')
        self.debug('upload_image:'+str(manifest)+'. manifest:'+str(upmanifest))
        return upmanifest

    def euca2ools_bundle_and_upload(self,
                                    file,
                                    arch,
                                    bucket,
                                    prefix=None,
                                    directory=None,
                                    kernel=None,
                                    ramdisk=None,
                                    block_device_map=[],
                                    product_codes=None,
                                    acl=None,
                                    location=None,
                                    ):
        raise NotImplemented('euca2ools_bundle_and_upload wrapper '
                             'not implemented yet')

    def euca2ools_register(self,
                           manifest,
                           name,
                           description=None,
                           arch=None,
                           kernel=None,
                           ramdisk=None,
                           root_device_name=None,
                           snapshot_id=None,
                           block_dev_map=[],
                           virtualization_type=None,
                           platform=None,
                           machine=None,
                           machine_credpath=None):
        machine = machine or self.worker_machine
        credpath = machine_credpath or self.credpath
        cmdargs = str(manifest) + " -n " + str(name)
        emi = None
        if description:
            cmdargs += ' -d ' + str(description)
        if arch:
            cmdargs += ' -a ' + str(arch)
        if kernel:
            cmdargs += ' --kernel ' + str(kernel)
        if ramdisk:
            cmdargs += ' --ramdisk ' + str(ramdisk)
        if root_device_name:
            cmdargs += ' --root-device-name ' + str(root_device_name)
        if snapshot_id:
            cmdargs += ' -s ' + str(snapshot_id)
        for dev in block_dev_map:
            cmdargs += ' -b ' + str(dev)
        if virtualization_type:
            cmdargs += ' --virtualization-type ' + str(virtualization_type)
        if platform:
            cmdargs += ' --platform ' + str(platform)

        if credpath is not None:
            cmd = 'source ' + str(credpath) + '/eucarc && euca-register '\
                  + str(cmdargs)
        else:
            skey = self.tester.get_secret_key()
            akey = self.tester.get_access_key()
            cmd = ('euca-upload-register -a ' + str(akey) +
                   ' -s ' + str(skey) + str(cmdargs))
        out = machine.sys(cmd=cmd, code=0)
        for line in out:
            if re.search('IMAGE',line):
                emi = line.split().pop().strip()
        assert emi, 'Invalid emi value: "' + str(emi) + '"'
        self.tester.test_resources["images"].append(self.tester.ec2.get_all_images([emi])[0])
        return emi

    def euca2ools_download_bundle(self,
                                  bucket,
                                  manifest=None,
                                  prefix=None,
                                  directory=None,
                                  image_name=None,
                                  machine=None,
                                  machine_credpath=None,
                                  ):
        machine = machine or self.worker_machine

        credpath = machine_credpath or self.credpath
        cmdargs = " -b " + str(bucket)
        if manifest:
            cmdargs += " -m " + str(manifest)
        if prefix:
            cmdargs += " -p " + str(prefix)
        if directory:
            cmdargs += " -d " + str(directory)
        if credpath is not None:
            cmd = 'source ' + str(credpath) + \
                  '/eucarc && euca-download-bundle '\
                  + str(cmdargs)
        else:
            skey = self.tester.get_secret_key()
            akey = self.tester.get_access_key()
            cmd = ('euca-download-bundle -a ' + str(akey) +
                   ' -s ' + str(skey) + str(cmdargs))
        out = machine.sys(cmd=cmd, code=0)



        raise NotImplemented('euca2ools_download_bundle wrapper '
                             'not implemented yet')

    def euca2ools_download_and_unbundle(self,
                                        bucket,
                                        manifest=None,
                                        prefix=None,
                                        directory=None,
                                        maxbytes=None,
                                        ):
        raise NotImplemented('euca2ools_download_and_unbundle wrapper'
                             ' not implemented yet')


    def euca2ools_import_volume(self,
                                import_file,
                                bucket,
                                zone,
                                format,
                                size=None,
                                presigned_manifest_url=None,
                                prefix=None,
                                days=None,
                                no_upload=None,
                                description=None,
                                s3_url=None,
                                ec2_url=None,
                                owner_sak=None,
                                owner_akid=None,
                                security_token=None,
                                machine=None,
                                machine_credpath=None,
                                misc=None):
        '''
        Note: Under normal conditions this will create a volume that may
        not be available to the returned task object immediately. The volume
        id will be available later by using task.update() on the returned
        task object. For this reason the volume is not being added to
        the tester's resources list here.
        '''
        machine = machine or self.worker_machine
        credpath = machine_credpath or self.credpath
        cmdargs = str(import_file) + " -b " + str(bucket) + \
                  " -z " + str(zone) + " -f " + str(format) + \
                  " --show-empty-fields "
        emi = None
        if description:
            cmdargs += ' -d ' + str(description)
        if size:
            cmdargs += ' -s ' + str(size)
        if presigned_manifest_url:
            cmdargs += ' --manifest-url ' + str(presigned_manifest_url)
        if prefix:
            cmdargs += " --prefix " + str(prefix)
        if days:
            cmdargs += " -x " + str(days)
        if no_upload:
            cmdargs += " --no-upload "
        if s3_url:
            cmdargs += " --s3-url " + str(s3_url)
        if ec2_url:
            cmdargs += " -U " + str(ec2_url)
        if owner_sak:
            cmdargs += " -w " + str(owner_sak)
        if owner_akid:
            cmdargs += " -o " + str(owner_akid)
        if security_token:
            cmdargs += " --security-token " + str(security_token)
        if misc:
            cmdargs += misc
        if credpath is not None:
            cmd = ('source ' + str(credpath) +
                   '/eucarc && euca-import-volume ' +
                   str(cmdargs))
        else:
            cmd = 'euca-upload-import-volume '
            # if keys were not already applied to args, fetch and populate
            # them now.
            if not owner_sak:
                owner_sak = self.tester.get_secret_key()
                cmdargs += " -w " + str(owner_sak)
            if not owner_akid:
                owner_akid = self.tester.get_access_key()
                cmdargs += " -o " + str(owner_akid)
            cmd += str(cmdargs)
        out = machine.sys(cmd=cmd, code=0)
        for line in out:
            lre = re.search('import-vol-\w{8}', line)
            if lre:
                taskid = lre.group()
        self.debug('Import taskid:' + str(taskid))
        #check on system using describe...
        task = self.tester.get_conversion_task(taskid=taskid)
        assert task, 'Task not found in describe conversion tasks. "{0}"'\
            .format(str(taskid))
        self.tester.test_resources['volumes']
        return task


    def euca2ools_import_instance(self,
                                import_file,
                                bucket,
                                zone,
                                format,
                                instance_type,
                                arch,
                                platform,
                                size=None,
                                keypair=None,
                                presigned_manifest_url=None,
                                prefix=None,
                                days=None,
                                no_upload=None,
                                description=None,
                                group=None,
                                s3_url=None,
                                ec2_url=None,
                                image_size=None,
                                private_addr=None,
                                shutdown_behavior=None,
                                owner_sak=None,
                                owner_akid=None,
                                security_token=None,
                                machine=None,
                                machine_credpath=None,
                                misc=None):
        machine = machine or self.worker_machine
        credpath = machine_credpath or self.credpath
        cmdargs = str(import_file) + " -b " + str(bucket) + \
                  " -z " + str(zone) + " -f " + str(format) + \
                  " -t " + str(instance_type) + " -a " + str(arch) + \
                  " -p " + str(platform) + \
                  " --show-empty-fields "
        emi = None
        if description:
            cmdargs += ' -d ' + str(description)
        if group:
            cmdargs += ' -g ' + str(group)
        if keypair:
            cmdargs += ' --key ' + str(keypair)
        if size:
            cmdargs += ' -s ' + str(size)
        if presigned_manifest_url:
            cmdargs += ' --manifest-url ' + str(presigned_manifest_url)
        if prefix:
            cmdargs += " --prefix " + str(prefix)
        if days:
            cmdargs += " -x " + str(days)
        if no_upload:
            cmdargs += " --no-upload "
        if s3_url:
            cmdargs += " --s3-url " + str(s3_url)
        if ec2_url:
            cmdargs += " -U " + str(ec2_url)
        if image_size:
            cmdargs += " --image-size " + str(image_size)
        if private_addr:
            cmdargs += " --private-ip-address"
        if shutdown_behavior:
            cmdargs += " --instance-initiated-shutdown-behavior " + \
                        str(shutdown_behavior)
        if owner_sak:
            cmdargs += " -w " + str(owner_sak)
        if owner_akid:
            cmdargs += " -o " + str(owner_akid)
        if security_token:
            cmdargs += " --security-token " + str(security_token)
        if misc:
            cmdargs += misc
        if credpath is not None:
            cmd = ('source ' + str(credpath) +
                   '/eucarc && euca-import-instance ' +
                   str(cmdargs))
        else:
            cmd = 'euca-upload-import-volume '
            # if keys were not already applied to args, fetch and populate
            # them now.
            if not owner_sak:
                owner_sak = self.tester.get_secret_key()
                cmdargs += " -w " + str(owner_sak)
            if not owner_akid:
                owner_akid = self.tester.get_access_key()
                cmdargs += " -o " + str(owner_akid)
            cmd += str(cmdargs)
        out = machine.sys(cmd=cmd, code=0)
        taskid = None
        for line in out:
            lre = re.search('import-i-\w{8}', line)
            if lre:
                taskid = lre.group()
        if not taskid:
            raise RuntimeError('Could not find a task id in output from cmd:'
                               + str(cmd))
        self.debug('Import taskid:' + str(taskid))
        #check on system using describe...
        task = self.tester.get_conversion_task(taskid=taskid)
        assert task, 'Task not found in describe conversion tasks. "{0}"'\
            .format(str(taskid))
        return task

    def _parse_euca2ools_conversion_task_output(self, output):
        splitlist = []
        retlist = []
        for line in output:
            splitlist.extend(line.split('\t'))
        if len(splitlist) % 2:
            raise ValueError("Conversion task output:\n" + str(output) +
                             '\nParsed euca2ools conversion task output does '
                             'not have an even number of fields to parse')
        #Build out a dict based upon the key value-ish format of the output
        task = ConversionTask()
        for i in xrange(0, len(splitlist)):
            if i % 2:
                task.endElement(name=lastkey,value=splitlist[i])
            else:
                lastkey = str(splitlist[i]).lower()
                if lastkey == 'tasktype':
                    if task:
                        retlist.append(task)
                    task = ConversionTask
        return retlist

    def _generate_unique_bucket_name_from_manifest(self,manifest, unique=True):
        mlist = str(manifest.replace('.manifest.xml', '')).split('/')
        basename = mlist[len(mlist)-1].replace('_', '').replace('.', '')
        if unique:
            return self._get_unique_bucket_name(basename)
        return basename

    def _get_unique_bucket_name(self, basename, id='test', start=0):
        bx=start
        bname = basename
        while self.tester.get_bucket_by_name(bname) is not None:
            bx += 1
            bname = basename+str(id)+str(bx)
        return bname


    def create_working_dir_on_worker_machine(self, path, overwrite=False):
        path = str(path)
        if self.worker_machine.is_file_present(path):
            if not overwrite:
                raise Exception('Dir found on:' +
                                str(self.worker_machine.hostname) +
                                ',"' + path + '".\n' +
                                'Either remove conflicting files, '
                                'use "filepath" option or "overwrite"')
        else:
            self.worker_machine.sys('mkdir -p ' + path, code=0)


    def _bundle_status_cb(self, buf, cmdtimeout, parttimeout, starttime,
                         lasttime, check_image_stage):
        ret = SshCbReturn(stop=False)
        #if the over timeout or the callback interval has expired,
        # then return stop=true
        #interval timeout should not be hit due to the setting of the
        # timer value, but check here anyways
        
        if (cmdtimeout != 0) and ( int(time.time()-starttime) > cmdtimeout):
            self.debug('bundle_status_cb command timed out after ' +
                       str(cmdtimeout)+' seconds')
            ret.statuscode=-100 
            ret.stop = True
            return ret
        if not check_image_stage:
            ret.settimer = parttimeout
            if (parttimeout != 0 and lasttime != 0) and \
                    (int(time.time()-lasttime) > parttimeout):
                self.debug('bundle_status_cb inter-part time out after ' +
                           str(parttimeout) + ' seconds')
                ret.statuscode=-100 
                ret.stop = True
                return ret
    
        if re.search('[P|p]art:',buf):
            sys.stdout.write("\r\x1b[K"+str(buf).strip())
            sys.stdout.flush()
            check_image_stage=False
        else: 
            #Print command output and write to ssh.cmd['output'] buffer
            ret.buf = buf
            self.debug(str(buf))
        #Command is still going, reset timer thread to intervaltimeout,
        # provide arguments for  next time this is called from ssh cmd.
        ret.stop = False
        ret.nextargs =[cmdtimeout, parttimeout, starttime,
                       time.time(), check_image_stage]
        return ret

    def create_emi(self,
                    url,
                    machine=None,
                    bucketname=None,
                    machine_credpath=None,
                    debug=False,
                    prefix=None,
                    kernel=None,
                    ramdisk=None,
                    architecture=None,
                    block_device_mapping=[],
                    destpath=None,
                    root_device_name=None,
                    description=None,
                    virtualization_type=None,
                    platform=None,
                    name=None,
                    interbundle_timeout=120,
                    upload_timeout=0,
                    uniquebucket=True,
                    wget_user=None,
                    wget_password=None,
                    wget_retryconn=True,
                    filepath=None,
                    bundle_manifest=None,
                    upload_manifest=None,
                    time_per_gig=300,
                    tagname=None,
                    overwrite=False
                    ):
        start = time.time()
        filesize = None
        destpath = destpath or self.destpath
        destpath = str(destpath)
        if not destpath.endswith('/'):
            destpath += '/'
        if url:
            filename = str(url).split('/')[-1]
            destpath = destpath + str(filename.replace('.','_'))
        self.debug('create_emi_from_url:' + str(url) + ", starting...")
        if filepath is None and bundle_manifest is None and \
                        upload_manifest is None:
            filepath = destpath + "/" + str(filename)
            self.create_working_dir_on_worker_machine(path=destpath,
                                                      overwrite=overwrite)

            self.debug('Downloading image to ' + str(machine) + ':' +
                       str(filepath) + ', url:' + str(url))
            filesize = self.wget_image(url,
                                       destpath=destpath,
                                       machine=machine,
                                       user=wget_user,
                                       password=wget_password,
                                       retryconn=wget_retryconn,
                                       time_per_gig=time_per_gig)
            
        self.status('create_emi_from_url: Image downloaded to machine, '
                    'now bundling image...')
        if bundle_manifest is None and upload_manifest is None:
            bundle_manifest = self.euca2ools_bundle_image(
                filepath,
                machine=machine,
                machine_credpath=machine_credpath,
                prefix=prefix,
                kernel=kernel,
                ramdisk=ramdisk,
                block_device_mapping=block_device_mapping,
                destination=destpath,
                debug=debug,
                interbundle_timeout=interbundle_timeout,
                time_per_gig=time_per_gig)
        
        self.status('create_emi_from_url: Image bundled, now uploading...')
        if upload_manifest is None:
            upload_manifest = self.euca2ools_upload_bundle(
                bundle_manifest,
                machine=machine,
                bucketname=bucketname,
                machine_credpath=machine_credpath,
                debug=debug,
                interbundle_timeout=interbundle_timeout,
                timeout=upload_timeout,
                uniquebucket=uniquebucket)
        
        self.status('create_emi_from_url: Now registering...')
        if name is None:
            name = upload_manifest.split('/').pop().rstrip('manifest.xml')
            name = "".join(re.findall(r"\w", name))
            name += '-' + str(int(time.time()))
        try:
            self.tester.get_emi(name='name')
        except:
            name += 'X'
        emi = self.euca2ools_register(
            manifest=upload_manifest,
            name=name,
            description=description,
            arch=architecture,
            kernel=kernel,
            ramdisk=ramdisk,
            root_device_name=root_device_name,
            block_dev_map=block_device_mapping,
            virtualization_type=virtualization_type,
            platform=platform)
        self.debug('euca2ools_register returned:"' + str(emi) +
                   '", now verify it exists on the cloud...')

        #Verify emi exists on the system, and convert to boto obj...
        emi = self.tester.get_emi(emi)

        #Add tags that might have test use meaning...
        try:
            if filesize is not None:
                emi.add_tag('size', value= str(filesize))
            if url:
                emi.add_tag('source', value=(str(url)))
            emi.add_tag(tagname or 'eutester-created')
        except Exception, te:
            self.debug('Could not add tags to image:' + str(emi.id) +
                       ", err:" + str(te))
        elapsed= int(time.time()-start)
        self.status('create_emi_from_url: Done, image registered as:' +
                    str(emi.id) + ", after " + str(elapsed) + " seconds")
        return emi




#!/usr/bin/python

import time, sys
import optparse
import os
import select

parser = optparse.OptionParser(prog='write_volume.py', description='write data to file to verify data integrity')
parser.add_option('-f', dest='testfile', help="file to read", default="/root/testfile")
parser.add_option('-b', dest='bytes', help="bytes to write", default=1000)
parser.add_option('-t', dest='timed_run', help="Number of seconds to run this script, 0 will run forever", default=0)

(args, stuff) = parser.parse_args()
bytes = int(args.bytes)
testfile = args.testfile
timed_run = int(args.timed_run)
readrate = "0"
writerate = "0"
lr = ""
timeout = 2
procfile = "/proc/sys/vm/drop_caches"
print "Starting read/write test, bytes:" + str(bytes) + ", file:" + str(testfile)
time.sleep(1)
f = open(testfile, 'w')
start = time.time()
elapsed = 0
time_remaining = timed_run - elapsed
try:
    while ( timed_run == 0 or time_remaining > 0 ):
        elapsed = time.time() - start
        time_remaining = int(timed_run - elapsed)
        wlength = 0
        start = time.time()
        for x in xrange(0, bytes):
            debug_string = "\nWRITE_VALUE: ".ljust(15) + str(x).ljust(15) + "\n" + \
                           "WRITE_RATE: ".ljust(15) + writerate.ljust(15) + " Bytes/Sec" + "\n" + \
                           "LAST_READ: ".ljust(15) + str(lr).ljust(15) + "\n" + \
                           "READ_RATE: ".ljust(15) + str(readrate + " Bytes/Sec").rjust(15) + "\n" + \
                           "TIME_REMAINING: ".ljust(15) + str(time_remaining)
            #print "\r\x1b[K Writing:" + str(x) + ", Rate:" + writerate + " bytes/sec, Lastread:" \
            #      + str(lr) + ", READ Rate:" + readrate + " bytes/sec",
            print "\r\x1b[K " + debug_string,
            sys.stdout.flush()
            xstr = x, x, x, x, x, x, x, x, x, x, x, x, x, x, x, x, x, x, x, x, x
            xstr = str(xstr).lstrip('(').rstrip(')') + "\n"
            f.write(xstr)
            wlength += len(str(xstr))
            if x and not x % 1:
                f.flush()
                os.fsync(f.fileno())
                time.sleep(.001)
                elapsed = time.time() - start
                writerate = str("%.2f" % (wlength / elapsed))
            f.flush()
            os.fsync(f.fileno())
        elapsed = time.time() - start
        writerate = str("%.2f" % (wlength / elapsed))
        elapsed = time.time() - start
        f.close()
        #Now do the read
        readin = ''
        leftover = ''
        length = 0
        last = 0
        proc = open(procfile, 'w')
        proc.write('3')
        proc.flush()
        proc.close()
        #f = os.open(testfile, os.O_DIRECT | os.O_RD)
        f = open(testfile, 'r')
        readstart = start = time.time()
        while True:
            reads, _, _ = select.select([f.fileno()], [], [], 0)
            elapsed = int(time.time() - start)
            if elapsed >= timeout:
                raise Exception("Could not read from file in timeout:" + str(elapsed))
            time.sleep(.1)
            if len(reads) > 0:
                start = time.time()
                readin = f.read(1024)
                if not readin:
                    break
                while readin:
                    if not readin:
                        break
                    length += len(readin)
                    readin = leftover + readin
                    #add any data not contained within a complete newline from last read() to this cycle...
                    leftover = readin.endswith('\n')
                    lines = readin.splitlines()
                    #if our last line wasn't a complete new line,save it for next read cycle...
                    if not leftover:
                        leftover = lines.pop()
                    for x in lines:
                        #print "\r\x1b[K Reading:" + str(x) + ",",
                        sys.stdout.flush()
                        cur = int(x.split(',')[0])
                        lr = str(cur)
                        if cur != 0 and cur != (last + 1):
                            raise Exception('bad incremented in value, last:' + str(last) + " vs cur:" + str(cur))
                        last = cur
                        cur = 0
                    readin = f.read(1024)
        if length != wlength:
            raise Exception(
                "Read length:" + str(length) + " != written length:" + str(wlength) + ",Diff:" + str(length - wlength))
        elapsed = time.time() - start
        readrate = str("%.2f" % (length / elapsed))
        rbuf = '\n\nREAD Rate:' + readrate + " bytes/sec\n\n"
        f.close()
        f = open(testfile, 'w')
finally:
    f.close()



'''
Created on Nov 16, 2011
@author: clarkmatthew



Simple Example script using the eutester web ui classes.

The intention of this script is to download and unzip credentials to a given file by 
interacting with the webui. 

-First a Eucaweb object is created which gives us a selenium interface to work with. 
In this case we do not need a eutester connection to  interact with the cloud outside the webui. 

-A euwebuser is created  with an account name, user name, and password. 
There are webmethods to create new users, but in this script we expect this user
already exists and has previously logged into the webui. 

-To simply access to predefined xpath variables, a euwebglobals object is created. 

-The script navigates to the webui login page, the user logs in, and downloads credentials to a given filename. 

'''


from eucaweb import Eucaweb, euwebuser, euwebaccount,euwebglobals, webtestcase
import time


    
    
def testrun():
    

        #New User to be added username will be randomly generated account is derived from acctadmin above
        user = euwebuser.Euwebuser(passwd="password", account="eucalyptus", user="admin")
        user.printUser()

        wg = euwebglobals.Webui_globals()
        

        #gui = eucawebdriver.Euguid(host="localhost",browser="FIREFOX",clc="192.168.51.9",needEutester=False)
        gui.goLoginPage()
        gui.login(user.account, user.user, user.passwd)
        time.sleep(5)
        gui.downloadCredentials(user.user, user.account, timeout=10, callBackMethod = testMethod, force = False)
     
        time.sleep(30)
        
        gui.tearDown(0)
        

def testMethod(filename):
    creddir= '/tmp/credentialtests'
    print "this is our test method we got filename("+filename+")"
    gui.unzipCredentialsToDir(filename, creddir)
    gui.sourceEucarcFromDir(creddir)
    
    
    
    

if __name__ == '__main__':
    gui = Eucaweb(host="localhost",browser="FIREFOX",clc="192.168.51.9",needEutester=False)
    testrun()
    print "this test is done"
        


#!/usr/bin/python

from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase
from eutester.machine import Machine


class ConfigureLoadBalancer(EutesterTestCase):
    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("--img-repo")
        self.get_args()
        # Setup basic eutester object
        self.tester = Eucaops( config_file=self.args.config,password=self.args.password)

    def clean_method(self):
        pass

    def ConfigureELB(self):
        """
        This is where the test description goes
        """
        clcs = self.tester.get_component_machines("clc")
        if len(clcs) == 0:
            raise Exception("Unable to find a CLC")
        first_clc = clcs[0]
        assert isinstance(first_clc,Machine)
        first_clc.add_repo(url=self.args.img_repo, name="EucaLoadBalancer")
        first_clc.install("eucalyptus-load-balancer-image")
        first_clc.sys("source " + self.tester.credpath  + "/eucarc && euca-install-load-balancer --install-default" , code=0)

if __name__ == "__main__":
    testcase = ConfigureLoadBalancer()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list
    list = testcase.args.tests or ["ConfigureELB"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)


#!/usr/bin/env python
#
#
# Description:  This script encompasses test cases/modules concerning instance specific behavior and
#               features for Eucalyptus.  The test cases/modules that are executed can be 
#               found in the script under the "tests" list.
import re

import time
from eucaops import Eucaops
from eutester.euinstance import EuInstance
from eutester.eutestcase import EutesterTestCase
import os
import random
from collections import namedtuple

class ReportingBasics(EutesterTestCase):
    def __init__(self, config_file=None, password=None):
        self.setuptestcase()
        # Setup basic eutester object
        self.tester = Eucaops( config_file=config_file, password=password)
        self.tester.poll_count = 120

        ### Add and authorize a group for the instance
        self.group = self.tester.add_group(group_name="group-" + str(time.time()))
        self.tester.authorize_group_by_name(group_name=self.group.name )
        self.tester.authorize_group_by_name(group_name=self.group.name, port=-1, protocol="icmp" )
        ### Generate a keypair for the instance
        self.keypair = self.tester.add_keypair( "keypair-" + str(time.time()))
        self.keypath = '%s/%s.pem' % (os.curdir, self.keypair.name)
        self.image = self.tester.get_emi(root_device_type="instance-store")
        self.reservation = None
        self.volume = None
        self.bucket = None
        self.private_addressing = False
        zones = self.tester.ec2.get_all_zones()
        self.zone = random.choice(zones).name
        self.clean_method = self.cleanup
        self.cur_time = str(int(time.time()))
        date_fields = time.localtime()
        self.date = str(date_fields.tm_year) + "-" + str(date_fields.tm_mon) + "-31"
        clcs = self.tester.get_component_machines("clc")
        if len(clcs) is 0:
            raise Exception("No CLC found")
        else:
            self.clc = clcs[0]
        poll_interval = 1
        write_interval = 1
        size_time_size_unit = "MB"
        size_time_time_unit = "MINS"
        size_unit = "MB"
        time_unit = "MINS"
        self.modify_property(property="reporting.default_poll_interval_mins",value=poll_interval)
        self.modify_property(property="reporting.default_write_interval_mins",value=write_interval)
        self.modify_property(property="reporting.default_size_time_size_unit",value=size_time_size_unit)
        self.modify_property(property="reporting.default_size_time_time_unit",value=size_time_time_unit)
        self.modify_property(property="reporting.default_size_unit",value=size_unit)
        self.modify_property(property="reporting.default_time_unit",value=time_unit)

    def cleanup(self):
        if self.reservation:
            self.assertTrue(self.tester.terminate_instances(self.reservation), "Unable to terminate instance(s)")
        if self.volume:
            self.tester.delete_volume(self.volume)
        if self.bucket:
            self.tester.clear_bucket(self.bucket)
        self.tester.delete_group(self.group)
        self.tester.delete_keypair(self.keypair)
        os.remove(self.keypath)

    def instance(self):
        self.reservation = self.tester.run_instance(self.image, keypair=self.keypair.name, group=self.group.name, zone=self.zone)
        file_size_in_mb = 500
        for instance in self.reservation.instances:
            assert isinstance(instance, EuInstance)
            self.volume = self.tester.create_volume(zone=self.zone, size=4)
            device_path = instance.attach_volume(self.volume)
            instance.sys("mkfs.ext3 -F " + device_path)
            instance.sys("mount " + device_path + " /mnt")
            ### Write to root fs
            instance.sys("dd if=/dev/zero of=/tmp/test.img count=" + str(file_size_in_mb) + " bs=1M")
            ### Write to volume
            instance.sys("dd if=/dev/zero of=/mnt/test.img count=" + str(file_size_in_mb) + " bs=1M")

        self.tester.sleep(180)
        for instance in self.reservation.instances:
            report_output = self.generate_report("instance","csv", self.date)
            instance_lines = self.tester.grep(instance.id, report_output)
            for line in instance_lines:
                instance_data = self.parse_instance_line(line)
                #if not re.search( instance.id +",m1.small,1,9,0.2,0,0,0,0,93,200,0.2,0.0,0,1", line):
                if not re.match(instance_data.type, "m1.small"):
                    raise Exception("Failed to find proper output for " + str(instance) + " type. Received: " + instance_data.type )
                if not int(instance_data.number)  == 1:
                    raise Exception("Failed to find proper output for " + str(instance) + " number. Received: " + instance_data.number )
                if not int(instance_data.unit_time)  > 2 :
                    raise Exception("Failed to find proper output for " + str(instance) + " unit_time. Received: " + instance_data.unit_time )
                if not int(instance_data.disk_write)  > 1000:
                    raise Exception("Failed to find proper output for " + str(instance) + " disk_write. Received: " + instance_data.disk_write )
                if not int(instance_data.disk_time_write)  > 200:
                    raise Exception("Failed to find proper output for " + str(instance) + " disk_time_write. Received: " + instance_data.disk_time_write )


    def parse_instance_line(self, line):
        InstanceData = namedtuple('InstanceData', 'id type number unit_time cpu net_total_in net_total_out '
                                                'net_extern_in net_extern_out disk_read disk_write disk_iops_read '
                                                'disk_iops_write disk_time_read disk_time_write')
        values = line.split(",")
        return InstanceData(values[0],values[1],values[2],values[3],values[4],values[5],values[6],values[7],
                            values[8],values[9],values[10],values[11],values[12],values[13],values[14])

    def s3(self):
        self.bucket = self.tester.create_bucket(bucket_name="reporting-bucket-" + self.cur_time)
        key_size = 10
        self.tester.debug("Creating random " + str(key_size) + "MB of data")
        rand_string = self.tester.id_generator(size=1024*1024*10)
        self.tester.upload_object(self.bucket.name, "reporting-key" ,contents=rand_string)
        self.tester.sleep(120)
        report_output = self.generate_report("s3", "csv",self.date)
        bucket_lines = self.tester.grep(self.bucket.name, report_output)
        for line in bucket_lines:
            bucket_data = self.parse_bucket_line(line)
            if not int(bucket_data.size) == 10:
                raise Exception('Failed to find proper size for %s' % str(self.bucket))
            if not int(bucket_data.keys) == 1:
                raise Exception('Failed to find proper number of keys for %s' % str(self.bucket))
            if not int(bucket_data.unit_time) > 16:
                raise Exception('Failed to find proper amount of usage for %s' % str(self.bucket))

    def parse_bucket_line(self, line):
        BucketData = namedtuple('BucketData', 'name keys size unit_time')
        values = line.split(",")
        return BucketData(values[0],values[1],values[2],values[3] )

    def generate_report(self, type, format, end_date):
        return self.clc.sys("source " + self.tester.credpath + "/eucarc && eureport-generate-report -t " +
                    str(type) +" -f " + str(format) + " -e " + str(end_date) )

    def modify_property(self, property, value):
        """
        Modify a eucalyptus property through the command line euca-modify-property tool
        property        Property to modify
        value           Value to set it too
        """
        command = "source " + self.tester.credpath + "/eucarc && " + self.tester.eucapath + "/usr/sbin/euca-modify-property -p " + str(property) + "=" + str(value)
        if self.clc.found(command, property):
            self.debug("Properly modified property " + property)
        else:
            raise Exception("Setting property " + property + " failed")


if __name__ == "__main__":
    testcase = EutesterTestCase()

    #### Adds argparse to testcase and adds some defaults args
    testcase.setup_parser()

    ### Get all cli arguments and any config arguments and merge them
    testcase.get_args()

    ### Instantiate an object of your test suite class using args found from above
    instance_basics_tests = testcase.do_with_args(ReportingBasics)

    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    list = testcase.args.tests or ["instance"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( instance_basics_tests.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    testcase.run_test_case_list(unit_list)


